@@ -1,7 +1,6 @@
 """Various constants, enums, and flags to aid readability."""
 
 from enum import Enum, IntFlag, auto, unique
-from typing import Dict, Tuple
 
 
 class StrEnum(str, Enum):  # Once we are on Python 3.11+: enum.StrEnum
@@ -97,9 +96,9 @@ class UserAccessPermissions(IntFlag):
         """Check if the given reserved name defaults to 1 = active."""
         return name not in {"R1", "R2"}
 
-    def to_dict(self) -> Dict[str, bool]:
+    def to_dict(self) -> dict[str, bool]:
         """Convert the given flag value to a corresponding verbose name mapping."""
-        result: Dict[str, bool] = {}
+        result: dict[str, bool] = {}
         for name, flag in UserAccessPermissions.__members__.items():
             if UserAccessPermissions._is_reserved(name):
                 continue
@@ -107,7 +106,7 @@ class UserAccessPermissions(IntFlag):
         return result
 
     @classmethod
-    def from_dict(cls, value: Dict[str, bool]) -> "UserAccessPermissions":
+    def from_dict(cls, value: dict[str, bool]) -> "UserAccessPermissions":
         """Convert the verbose name mapping to the corresponding flag value."""
         value_copy = value.copy()
         result = cls(0)
@@ -145,14 +144,6 @@ class Resources:
     PROPERTIES = "/Properties"  # dictionary, optional
 
 
-class Ressources:  # deprecated
-    """
-    Use :class: `Resources` instead.
-
-    .. deprecated:: 5.0.0
-    """
-
-
 class PagesAttributes:
     """§7.7.3.2 of the 1.7 and 2.0 reference."""
 
@@ -245,6 +236,7 @@ class FilterTypes(StrEnum):
     CCITT_FAX_DECODE = "/CCITTFaxDecode"  # abbreviation: CCF
     DCT_DECODE = "/DCTDecode"  # abbreviation: DCT
     JPX_DECODE = "/JPXDecode"
+    JBIG2_DECODE = "/JBIG2Decode"
 
 
 class FilterTypeAbbreviations:
@@ -450,7 +442,7 @@ class FieldDictionaryAttributes:
         """Ch"""
 
     @classmethod
-    def attributes(cls) -> Tuple[str, ...]:
+    def attributes(cls) -> tuple[str, ...]:
         """
         Get a tuple of all the attributes present in a Field Dictionary.
 
@@ -477,7 +469,7 @@ class FieldDictionaryAttributes:
         )
 
     @classmethod
-    def attributes_dict(cls) -> Dict[str, str]:
+    def attributes_dict(cls) -> dict[str, str]:
         """
         Get a dictionary of attribute keys and their human-readable names.
 
@@ -509,7 +501,7 @@ class CheckboxRadioButtonAttributes:
     Opt = "/Opt"  # Options, Optional
 
     @classmethod
-    def attributes(cls) -> Tuple[str, ...]:
+    def attributes(cls) -> tuple[str, ...]:
         """
         Get a tuple of all the attributes present in a Field Dictionary.
 
@@ -525,7 +517,7 @@ class CheckboxRadioButtonAttributes:
         return (cls.Opt,)
 
     @classmethod
-    def attributes_dict(cls) -> Dict[str, str]:
+    def attributes_dict(cls) -> dict[str, str]:
         """
         Get a dictionary of attribute keys and their human-readable names.
 
@@ -721,3 +713,52 @@ class ImageType(IntFlag):
     DRAWING_IMAGES = auto()
     ALL = XOBJECT_IMAGES | INLINE_IMAGES | DRAWING_IMAGES
     IMAGES = ALL  # for consistency with ObjectDeletionFlag
+
+
+_INLINE_IMAGE_VALUE_MAPPING = {
+    "/G": "/DeviceGray",
+    "/RGB": "/DeviceRGB",
+    "/CMYK": "/DeviceCMYK",
+    "/I": "/Indexed",
+    "/AHx": "/ASCIIHexDecode",
+    "/A85": "/ASCII85Decode",
+    "/LZW": "/LZWDecode",
+    "/Fl": "/FlateDecode",
+    "/RL": "/RunLengthDecode",
+    "/CCF": "/CCITTFaxDecode",
+    "/DCT": "/DCTDecode",
+    "/DeviceGray": "/DeviceGray",
+    "/DeviceRGB": "/DeviceRGB",
+    "/DeviceCMYK": "/DeviceCMYK",
+    "/Indexed": "/Indexed",
+    "/ASCIIHexDecode": "/ASCIIHexDecode",
+    "/ASCII85Decode": "/ASCII85Decode",
+    "/LZWDecode": "/LZWDecode",
+    "/FlateDecode": "/FlateDecode",
+    "/RunLengthDecode": "/RunLengthDecode",
+    "/CCITTFaxDecode": "/CCITTFaxDecode",
+    "/DCTDecode": "/DCTDecode",
+    "/RelativeColorimetric": "/RelativeColorimetric",
+}
+
+_INLINE_IMAGE_KEY_MAPPING = {
+    "/BPC": "/BitsPerComponent",
+    "/CS": "/ColorSpace",
+    "/D": "/Decode",
+    "/DP": "/DecodeParms",
+    "/F": "/Filter",
+    "/H": "/Height",
+    "/W": "/Width",
+    "/I": "/Interpolate",
+    "/Intent": "/Intent",
+    "/IM": "/ImageMask",
+    "/BitsPerComponent": "/BitsPerComponent",
+    "/ColorSpace": "/ColorSpace",
+    "/Decode": "/Decode",
+    "/DecodeParms": "/DecodeParms",
+    "/Filter": "/Filter",
+    "/Height": "/Height",
+    "/Width": "/Width",
+    "/Interpolate": "/Interpolate",
+    "/ImageMask": "/ImageMask",
+}
